

DROP TYPE tsalecost CASCADE;
CREATE TYPE tsalecost AS (
	unit	VARCHAR,
	cost	NUMERIC(19,4)
);



DROP FUNCTION fSale_ItemCost(VARCHAR, VARCHAR, VARCHAR, VARCHAR, INT);
CREATE OR REPLACE FUNCTION fSale_ItemCost(
	__ItemID	VARCHAR,
	__WhID		VARCHAR,
	__PID		VARCHAR,
	__SaleNo	VARCHAR,	-- optional
	__SaleDNo	INT		-- optional
) RETURNS SETOF tsalecost AS
$BODY$
DECLARE
	cur1					REFCURSOR;
	_retval					tsalecost%ROWTYPE;
	_PID					VARCHAR;
	_force_try_to_match_any_best_cost	BOOLEAN;
BEGIN
	_force_try_to_match_any_best_cost:=TRUE;	-- default=FALSE, Ubah bila perlu

	_retval:=(NULL, 0);
	_PID:=__PID;

/**
	IF (_PID IS NULL) THEN 
		_PID:='';
	END IF;
**/

	IF (__ItemID IS NULL) THEN 
		RETURN NEXT _retval;
		RETURN;
	END IF;

	IF (__SaleNo IS NOT NULL) THEN
		IF (__SaleDNo IS NOT NULL) THEN 
			SELECT INTO _retval.unit, _retval.cost unit, cost 
			FROM saled 
			WHERE saleno=__SaleNo AND saledno=__SaleDNo;
		ELSE
			SELECT INTO _retval.unit, _retval.cost unit, cost 
			FROM saled 
			WHERE saleno=__SaleNo AND itemid=__ItemID AND whid=__WhID AND pid=_PID;
		END IF;
	ELSE
		IF (__PID IS NOT NULL) THEN
			-- Baca dari SaleD yang terakhir
			SELECT INTO _retval.unit, _retval.cost 
				unit, cost 
			FROM saled d
			JOIN sale h ON h.saleno=d.saleno
			WHERE d.itemid=__ItemID 
			AND (__WhID IS NULL OR d.whid=__WhID)
			AND d.pid=_PID
			ORDER BY h.saledate DESC, h.saleno DESC LIMIT 1;

			IF (_retval.cost=0) THEN
				SELECT INTO 
					_retval.unit, _retval.cost d.cost, d.unit 
				FROM saled d
				JOIN sale h ON h.saleno=d.saleno
				WHERE d.itemid=__ItemID 
				AND (__WhID IS NULL OR d.whid=__WhID)
				AND d.pid=_PID
				AND d.cost>0
				ORDER BY h.saledate DESC, h.saleno DESC LIMIT 1;
			END IF;
		ELSE
			SELECT INTO 
				_retval.unit, _retval.cost unit, cost 
			FROM saled d
			JOIN sale h ON h.saleno=d.saleno
			WHERE d.itemid=__ItemID 
			AND (__WhID IS NULL OR d.whid=__WhID)
				-- sembarang PID 
			ORDER BY h.saledate DESC, h.saleno DESC LIMIT 1;
		END IF;
	END IF;

	-- Misal SRetD ter-set SaleNo, tapi mungkin SaleDNo gak diisi (bug prg lama), atau
	-- SaleNo-nya udah di utik2 (bug prg jg blm di lock)
	-- dan kita mesti ngeyel set HPP Returnya, maka dicariin berdasarkan PID aja
	IF ((_retval IS NULL OR _retval.cost=0) AND (_force_try_to_match_any_best_cost=TRUE)) THEN
		RAISE NOTICE '		RetVal=%, Smart searching Cost now', _retval;

		IF (_PID IS NOT NULL) THEN			-- codeblock dibawah ini ngopi dari block diatas
			SELECT INTO 
				_retval.unit, _retval.cost unit, cost 
			FROM saled d
			JOIN sale h ON h.saleno=d.saleno
			WHERE d.itemid=__ItemID 
			AND (__WhID IS NULL OR d.whid=__WhID)
			AND d.pid=_PID
			ORDER BY h.saledate DESC, h.saleno DESC LIMIT 1;

			IF (_retval.cost=0) THEN
				SELECT INTO 
					_retval.unit, _retval.cost unit, cost 
				FROM saled d
				JOIN sale h ON h.saleno=d.saleno
				WHERE d.itemid=__ItemID 
				AND (__WhID IS NULL OR d.whid=__WhID)
				AND d.pid=_PID
				AND d.cost>0
				ORDER BY h.saledate DESC, h.saleno DESC LIMIT 1;
			END IF;
		END IF;
	END IF;


	IF NOT FOUND THEN
		_retval:=(NULL, 0);
	END IF;

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


/******************** OVERLOAD *******************/

DROP FUNCTION fSale_ItemCost(VARCHAR, VARCHAR, INT);
CREATE OR REPLACE FUNCTION fSale_ItemCost(
	_ItemID		VARCHAR,
	_SaleNo		VARCHAR,
	_SaleDNo	INT
) RETURNS SETOF tsalecost AS 
$BODY$
DECLARE
	_Cost	DECIMAL(19,4);
	_retval	tsalecost%ROWTYPE;
BEGIN	
	_retval:=(NULL, 0);

	SELECT INTO 
		_retval.cost, _retval.unit
		cost, unit
	FROM saled
	WHERE saleno=_SaleNo AND saledno=_SaleDNo AND itemid=_ItemID;

	IF NOT FOUND THEN
		_retval:=(NULL, 0);
	END IF;

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spSRet_New (
	_SRetNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_CustID			VARCHAR(15);
	_ConsNo			VARCHAR(15);
	_SRetDate		DATE;
	_TermType		VARCHAR(1);
	_Total			NUMERIC(19,4);
	_CashID			VARCHAR(15);
	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;

	_SRetDNo		SMALLINT;
	_ItemID			VARCHAR(20);
	_WhID			VARCHAR(3);
	_PID			VARCHAR(25);
	_Qty			NUMERIC(15,4);
	_Unit			VARCHAR(5);
	_Conv			NUMERIC(15,4);
	_QtyX			NUMERIC(15,4);
	_Cost			NUMERIC(19,4);
	_ItemType		VARCHAR(5);

	_StockANo		VARCHAR(15);
	_StockANote		VARCHAR(255);
	_StockADNo		SMALLINT;

	_SRetQty		NUMERIC(15,4);
	_SRetQtyX		NUMERIC(15,4);
	_ItemsCost		NUMERIC(19,4);

	_CashAmt		NUMERIC(19,4);
	_CashANote		VARCHAR(255);

	_IsContra		BOOLEAN;
	_regval			VARCHAR(30);

	_RcvNo			VARCHAR(15);
	_CrcID			VARCHAR(5);

	_SpaidNo		INT2;
	_SpaidAmt		NUMERIC(19,4);
	_Bayar			NUMERIC(19,4);

	_Freight		DECIMAL(19,4);
	_Others			DECIMAL(19,4);
	_Disc			DECIMAL(19,4);
	_Count1			INT;
	_Count2			INT;
	_JnlNo1			VARCHAR(15);
	_JnlNo2			VARCHAR(15);
	_JnlNote		VARCHAR;
	_ListPrice		DECIMAL(19,4);
	_DebtAmt		DECIMAL(19,4);
	_ExcRate		DECIMAL(19,4);
	_DebtNote		VARCHAR(50);

	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
	_DeptID			VARCHAR(5);
	_PrjID			VARCHAR(5);

	cSRetD			REFCURSOR;
	cSRetD2			REFCURSOR;
	cSRet			REFCURSOR;
	cSRetAcc		REFCURSOR;

	_AccAsset		VARCHAR(10);
	_AccDisc		VARCHAR(10);
	_AccInv			VARCHAR(10);
	_AccCOGS		VARCHAR(10);
	_AccFreight		VARCHAR(10);
	_AccSRet		VARCHAR(10);
	_AccAmt			DECIMAL(19,4);    
	_MTaxAmt		DECIMAL(19,4);  
	_countDeli		INT;
	_UBDeliAmt		DECIMAL(19,4);
	_Round			INT;
	_RoundTax		INT;
	

/** buat SRetdeli **/
	cSRetDeli		REFCURSOR;
	_DeliNo			VARCHAR(15);
	_SaleNo			VARCHAR;
	_SaleDNo		SMALLINT;

	_unit1			VARCHAR;
	_unit2			VARCHAR;
	_unit3			VARCHAR;
	_conv2			DECIMAL;
	_conv3			DECIMAL;
	_listpriceAsli		DECIMAL;
	_QtyAsli		DECIMAL;
	_UnitAsli		VARCHAR;

	_qty_saled		DECIMAL;
	_qty_delid		DECIMAL;
	_qty_sretd		DECIMAL;

	cSale			REFCURSOR;
	cTax			REFCURSOR;

	_isDraft		BOOLEAN;
	_LockDate		DATE;

	_ItemIDCheck		VARCHAR;
	_SRetDTaxAmt		NUMERIC(19,4);
	_AccSRetTax		VARCHAR;
	_TotalTax		NUMERIC(19,4);
	_SelisihTax		NUMERIC(19,4);
	_i			INT;
	_Subtotal		NUMERIC(19,4);
	_SelisihBasePrice	NUMERIC(19,4);
	_TotalBasePrice		NUMERIC(19,4);
	_FisRate		NUMERIC(19,4);

	_SRetStatusYes		VARCHAR(1);

	_BTotal			NUMERIC(19,4); -- base currency total
	_BSubtotal		NUMERIC(19,4); -- base currency subtotal
	_MBFTaxAmt		NUMERIC(19,4); -- base currency taxamt
	_MBDiscAmt		NUMERIC(19,4); -- base currency discamt
	
	_TaxableAmt		NUMERIC(19,4);
	_TotalTaxAmt		NUMERIC(19,4);
	_BTotalTaxAmt		NUMERIC(19,4);
	_TotalDiscAmt		NUMERIC(19,4);
	_TotalDisc2Amt		NUMERIC(19,4);
	_DSubtotal		NUMERIC(19,4);
	_DBSubtotal		NUMERIC(19,4);
	
	_AccRateLR		VARCHAR; -- akun laba/rugi selisih kurs
	_RateLR			NUMERIC(19,4); -- nilai laba/rugi selisih kurs

	_CashCrcID		VARCHAR;
	_BoMID			bomd.bomid%TYPE;
	_StockAType		VARCHAR(5):='SRET';
	_InOut			CHAR(1);
	_DeliDNo		INT2;

	_SRetNote		VARCHAR;
	_SaleCost		tsalecost%ROWTYPE;
	_QtyA			NUMERIC(15,4);
	_QtyB			NUMERIC(15,4);
BEGIN
	_DeptID:=NULL;
	_PrjID:=NULL;
	_SRetStatusYes:='Y';
	_InOut:='O';

	SELECT INTO
		_isDraft, _PerID, _BranchID, _CustID, _SRetDate, _TermType, _BTotal, _Total,_CrcID, _CashID, 
		_CrtBy, _CrtDate, _UpdBy, _UpdDate, _ConsNo, _Freight, _MBDiscAmt, _Disc, _AccDisc, _AccFreight, _ExcRate, 
		_MBFTaxAmt, _MTaxAmt,_Others,_IsContra, _BSubtotal, _SubTotal, _FisRate, _SRetNote

		isDraft, PerID, BranchID, CustID, SRetDate, TermType, basetotal, Total, crcid, CashID, 
		CrtBy, CrtDate, UpdBy, UpdDate, ConsNo, freight, basediscamt, discamt, accsdisc, accfreight, excrate, 
		basftaxamt, TaxAmt,others,iscontra, basesubtotal, Subtotal, FisRate, sretnote
	FROM SRet
	WHERE SRetNo=_SRetNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='ReturJual #'||_SRetNo||' tidak ditemukan (spSRet_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;


	IF EXISTS (
		SELECT d.* 
		FROM sretd d 
		JOIN item i ON d.itemid=i.itemid
		WHERE d.unit!=i.unit1 AND d.unit!=COALESCE(i.unit2, '') 
		AND d.unit!=COALESCE(i.unit3, '') AND d.sretno=_SRetNo
	) THEN
		_retval.ErrNo:=10001;
		_retval.ErrMsg:=E'Sudah ada perubahan UNIT pada Master Item\nSilahkan re-Open transaksi yang ingin di-edit (spSRet_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_Round:=COALESCE(fReg('ROUND'), '0');
	_RoundTax:=COALESCE(fReg('ROUND_TAX'), '0');	

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='SRET';
	IF (_LockDate IS NOT NULL AND _LockDate>_SRetDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='ReturJual Melebihi Batas Tanggal Kunci Transaksi ! (spSRet_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _countDeli COUNT(*) FROM SRetD WHERE SRetNo=_SRetNo AND DeliNo IS NOT NULL AND DeliNo<>'';


	IF (_TermType='R' AND _IsContra=TRUE) THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='ReturJual sudah ada Penerimaan! (spSRet_New)';	
		RAISE EXCEPTION '%', _retval.ErrMsg;	
	END IF;
	
	OPEN cSRetDeli FOR
		SELECT delino, delidno, itemid, whid, pid, qty, unit 
		FROM sretd 
		WHERE sretno=_SRetNo 
		AND (delino IS NOT NULL AND delino<>'');
	LOOP	
		FETCH cSRetDeli INTO _DeliNo, _DeliDNo, _ItemID, _WhID, _PID, _Qty, _Unit;	
		EXIT WHEN NOT FOUND;

		SELECT INTO _qty_delid COALESCE(SUM(fQtyZToQty1(itemid, qty, unit)),0) FROM delid WHERE itemid=_ItemID AND pid=_PID AND delino=_DeliNo AND delidno=_DeliDNo;
		SELECT INTO _qty_sretd COALESCE(SUM(fQtyZToQty1(itemid, qty, unit)),0) FROM sretd WHERE itemid=_ItemID AND pid=_PID AND delino=_DeliNo AND delidno=_DeliDNo;

		IF _qty_sretd>_qty_delid THEN
			_retval.ErrNo:=50003;
			_retval.ErrMsg:='Error : Qty Retur Item:'||_ItemID||', Wh:'||_WhID||', PID:'||_PID||', Unit:'||_Unit||' tidak boleh lebih dari Qty pada Pengiriman (spSRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		-- Update isInvoice=TRUE, saat ini gak bisa diInvoice pecah/parsial, jadi ALL or None
		UPDATE Deli SET isInvoiced=TRUE WHERE DeliNo=_DeliNo;

		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Error Update Pengiriman#'||_DeliNo|| !' (spSRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cSRetDeli;

	_ItemsCost:=0;
/** 
	Generate StockA
	jika countDeli = 0, artinya ndak import dari deli sama sekali, jadi generate stocka 
	jika countDeli > 0, stockanya udah didelinya
**/
	IF (_countDeli=0) THEN
		OPEN cSRetD FOR
			SELECT 
				d.SRetDNo, d.ItemID, d.WhID, d.PID, d.Qty, d.Unit, d.QtyX, i.ItemType, d.SaleNo, d.SaleDNo
			FROM SRetD d JOIN Item i ON d.ItemID=i.ItemID
			WHERE d.SRetNo=_SRetNo;

			--semua HPP baca dari saled, gak peduli PACK, SERV dll
			--AND (i.IsStock=TRUE OR i.ItemType='PACK');	

		_StockADNo=0;
		_i:=0;
		LOOP
			FETCH cSRetD INTO _SRetDNo, _ItemID, _WhID, _PID, _Qty, _Unit, _QtyX, _ItemType, _SaleNo, _SaleDNo;
			EXIT WHEN NOT FOUND;
			_i:=_i+1;

			SELECT INTO _qty_saled COALESCE(SUM(fQtyZToQty1(itemid, qty, unit)),0) FROM saled WHERE itemid=_ItemID AND pid=_PID AND saleno=_SaleNo AND saledno=_SaleDNo;
			SELECT INTO _qty_sretd COALESCE(SUM(fQtyZToQty1(itemid, qty, unit)),0) FROM sretd WHERE itemid=_ItemID AND pid=_PID AND saleno=_SaleNo AND saledno=_SaleDNo;
--RAISE EXCEPTION '% vs %', _qty_sretd, _qty_saled;
			IF _qty_sretd>_qty_saled THEN
				_retval.ErrNo:=50003;
				_retval.ErrMsg:='Error : Qty Retur Item:'||_ItemID||', Wh:'||_WhID||', PID:'||_PID||', Unit:'||_Unit||' tidak boleh lebih dari Qty pada Penjualan (spSRet_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF (_i=1) THEN		-- generate StockA header
				_StockANo:=fTransNoGet('STOCKA');
				_StockANote='Retur Jual '||fBPName(_CustID)||':'||_SRetNo;
				SELECT INTO _spret * FROM spStockA_New(
					_StockANo,_CustID, 'SRET', _SRetDate, 'O', 'SRET', _SRetNo, _StockANote, TRUE, 
					_CrtBy, _CrtDate, _UpdBy, _UpdDate,
					_PerID, _BranchID, _DeptID, _PrjID								
				);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;		
			END IF;			

/** mbuh gak jelas kok moro2 ngupdate ConsD , mestine dikumpulin jadi 1 blok dgn yg lain

			IF (_PID IS NULL) THEN
				UPDATE ConsD SET QtyCRet=QtyCRet+_Qty, QtyXCRet=QtyXCRet+_QtyX 
				WHERE ConsNo=_ConsNo AND ItemID=_ItemID;
			ELSE
				UPDATE ConsD SET QtyCRet=QtyCRet+_Qty, QtyXCRet=QtyXCRet+_QtyX 
				WHERE ConsNo=_ConsNo AND ItemID=_ItemID 
				AND PID=_PID; 
			END IF;
**/

			_StockADNo=_StockADNo+1;

			IF (_ItemType='PACK') THEN

				SELECT INTO _BoMID BoMID FROM BoM WHERE ItemID=_ItemID;
				/**_StockADNo:=_StockADNo+1; dicomment karena sudah ada modifikasi di spstockad_new_bom_sret*/
				
				IF(_BomID IS NULL) THEN
					_retval.ErrNo:='50014';
					_retval.ErrMsg='Error tidak dapat menemukan B.o.M. untuk Item:'||_ItemID||'(spSRet_New)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
		
				SELECT INTO _spret * FROM spStockAD_New_BoM_SRet(
					_SRetDate, _StockAType, _StockANo, _StockADNo, _WhID, _ItemID, _PID, 
					_Qty, _Unit, _QtyX, _Cost, _DeptID, _PrjID, _CrcID, _SaleNo, _SaleDNo, _SRetNo
				);

				_StockADNo:=_spret.ErrNo;

				IF _spret.ErrNo<>0 AND _spret.ErrMsg<>'OK' THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;			
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				_SaleCost:=fSale_ItemCost(_ItemID, _SaleNo, _SaleDNo);
				_QtyA=fQtyZToQty1(_ItemID, 1, _SaleCost.Unit);
				_QtyB=fQtyZToQty1(_ItemID, 1, _Unit);
				_Cost:=(_QtyB/_QtyA)*_SaleCost.cost;

				_ItemsCost=_ItemsCost+(_Qty*_Cost);

				IF (_Cost IS NULL) THEN
					_retval.ErrNo:=50003;
					_retval.ErrMsg:='Error menghitung HPP Item:'||_ItemID||', Wh:'||_WhID||', PID:'||_PID||', Unit:'||_Unit||' (fUnitCost)(spSRet_New)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
		
				_Conv:=fItemUnit_Conv(_ItemID, _Unit);

				UPDATE SRetD SET Cost=_Cost, Conv=_Conv WHERE SRetNo=_SRetNo AND SRetDNo=_SRetDNo;
				IF NOT FOUND THEN
					_retval.ErrNo:=50010;
					_retval.ErrMsg:='Error UPDATE SRetD:Cost (spSRet_New)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

			ELSE
				_SaleCost:=fSale_ItemCost(_ItemID, _SaleNo, _SaleDNo);
				_QtyA=fQtyZToQty1(_ItemID, 1, _SaleCost.Unit);
				_QtyB=fQtyZToQty1(_ItemID, 1, _Unit);
				_Cost:=(_QtyB/_QtyA)*_SaleCost.cost;

				_ItemsCost=_ItemsCost+(_Qty*_Cost);

				IF (_Cost IS NULL) THEN
					_retval.ErrNo:=50003;
					_retval.ErrMsg:='Error menghitung HPP Item:'||_ItemID||', Wh:'||_WhID||', PID:'||_PID||', Unit:'||_Unit||' (fUnitCost)(spSRet_New)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				SELECT INTO _spret * FROM spStockAD_New(
					_SRetDate, _StockAType, _StockANo, _StockADNo, _WhID, 
					_ItemID, _PID, _Qty, _Unit, _QtyX, _Cost, _DeptID, _PrjID, TRUE, _SRetNo);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
		
				_Conv:=fItemUnit_Conv(_ItemID, _Unit);

				UPDATE SRetD SET Cost=_Cost, Conv=_Conv WHERE SRetNo=_SRetNo AND SRetDNo=_SRetDNo;
				IF NOT FOUND THEN
					_retval.ErrNo:=50010;
					_retval.ErrMsg:='Error UPDATE SRetD:Cost (spSRet_New)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

			END IF;


		END LOOP;
		CLOSE cSRetD;
	END IF;
	
/**
	OPEN cSRetD2 FOR
	SELECT d.SRetDNo, d.ItemID, d.WhID, d.PID, d.Qty, d.Unit, d.QtyX, d.SaleNo, d.SaleDNo
	FROM SRetD d JOIN Item i ON d.ItemID=i.ItemID
	WHERE 
		d.SRetNo=_SRetNo 
		AND ((i.ItemType='SERV') 
		OR (i.IsStock=FALSE AND i.IsAsm=FALSE));
	LOOP
		FETCH cSRetD2 INTO _SRetDNo, _ItemID, _WhID, _PID, _Qty, _Unit, _QtyX, _SaleNo, _SaleDNo;
		EXIT WHEN NOT FOUND;
		
		_Cost:=fUnitCost(_ItemID, _WhID, _PID, _Unit);
		IF (_Cost IS NULL) THEN
			_retval.ErrNo:=50005;
			_retval.ErrMsg:='Error menghitung HPP! Item:'||_ItemID||', Wh:-, PID:'||_PID||', Unit:'||_Unit||' (fUnitCost)(spSRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_retval.ErrNo:=50006;
		_retval.ErrMsg:='Error Update SRetD:Cost [non-stock-item](spSRet_New)';

		UPDATE SRetD SET Cost=_Cost WHERE SRetNo=_SRetNo AND SRetDNo=_SRetDNo;

		IF NOT FOUND THEN
			_retval.ErrNo:=50010;
			_retval.ErrMsg:='Error UPDATE SRetD:Cost (spSRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_ItemsCost=_ItemsCost+(_Qty*_Cost);
	END LOOP;
	CLOSE cSRetD2;

**/

	IF (_TermType='C') THEN
		_IsContra=TRUE;
	ELSE
		_IsContra=FALSE;
	END IF;

	UPDATE SRet SET CostTot=_ItemsCost, IsContra=_IsContra WHERE SRetNo=_SRetNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50010;
		_retval.ErrMsg:='Error UPDATE SRet:CostTot (spSRet_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	

	IF (_TermType='C' AND _ConsNo IS NULL) THEN
		IF (_CashID IS NULL) THEN
			_retval.ErrNo:=50001;
			_retval.ErrMsg:='Error, Kas Retur Penjualan belum diset ! (spSRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _CashCrcID crcid FROM cash WHERE cashid=_CashID;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Mata uang kas#'||_CashID||' belum diset (spSRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
 					
		IF (_CrcID!=_CashCrcID) THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Mata uang kas#'||_CashID||' tidak sama dengan mata uang penjualan (spSRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;		
		END IF;
		_CashANote='Retur #'||_SRetNo;
		_CashAmt=_Total*(-1);

		SELECT INTO _spret * FROM spCashA_New(
				_CashID, _CustID, 'SRET', _SRetDate, 'O', 'SRET', _SRetNo, 
			 _CashAmt, _CashANote, TRUE, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Error Update SRet:SRetStatus, IsContra (spSRet_New)';
		UPDATE SRet SET SRetStatus=_SRetStatusYes, IsContra=_IsContra WHERE SRetNo=_SRetNo;

	ELSE
      	IF (_ConsNo IS NOT NULL) THEN	
			SELECT INTO _spret * FROM spSetConsStatus(_ConsNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		ELSE
			_DebtAmt:=_Total*(-1);		--- penerimaan : potong piutang (-)
			_DebtNote:='Retur #'||_SRetNo;

			SELECT INTO _spret * FROM spBPA_New(
				_CustID, 'P',_CrcID, _SRetDate, 'SRET', _SRetNo, _DebtAmt, _DebtNote, TRUE, 
				_CrtBy, _CrtDate, _UpdBy, _UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID, _ExcRate, _FisRate								
			);

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;			
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;	-- IF (ConsNo IS NOT NULL)

	END IF;	-- Term=C && ConsNo IS NULL

	OPEN cSale FOR
		SELECT qty, unit, saleno, saledno, itemid
		FROM sretd 
		WHERE (saleno IS NOT NULL AND saleno<>'') AND SRetNo=_SRetNo;
	LOOP
		FETCH cSale INTO _Qty, _unit, _SaleNo, _SaleDNo, _ItemID;
		EXIT WHEN NOT FOUND;

			SELECT INTO _QtyAsli, _UnitAsli, _unit1, _unit2, _unit3, _conv2, _conv3
				qty, unit, unit1, unit2, unit3, conv2, conv3
			FROM saled 
			JOIN item ON saled.itemid=item.itemid
			WHERE saleno=_SaleNo AND saledno=_SaleDNo
			AND saled.itemid=_ItemID;

			IF (_UnitAsli<>_unit) THEN
				IF (_UnitAsli=_unit1) THEN
					IF (_Unit=_unit2) THEN	
						_Qty:=_Qty*_conv2;
					ELSIF (_Unit=_unit3) THEN
						_Qty:=_Qty*_conv3;
					END IF;
				ELSIF (_UnitAsli=_unit2) THEN
					IF (_Unit=_unit1) THEN
						_Qty:=_Qty/_conv2;
					ELSIF (_Unit=_unit3) THEN
						_Qty:=(_Qty*_conv3)/_conv2;
					END IF;
				ELSIF (_UnitAsli=_unit3) THEN
					IF (_Unit=_unit1) THEN
						_Qty:=_Qty/_conv3;
					ELSIF (_Unit=_unit2) THEN
						_Qty:=(_Qty*_conv2)/_conv3;
					END IF;
				END IF;
			END IF;

			IF (_Qty>_QtyAsli) THEN
				RAISE EXCEPTION '%', 'Qty ITEM:'||_ItemID||' melebihi penjualan no:'||_SaleNo;
			END IF;
	END LOOP;
	CLOSE cSale;


/* ====================================== AKUNTANSI ==================================== */
	IF isAccInstalled() THEN
		_JnlNo1:=fTransNoGet('JURNAL');
		IF (_SRetNote IS NULL OR _SRetNote='') THEN
			_JnlNote:= 'Retur Jual '||fBPName(_CustID)||':'||_SRetNo;
		ELSE
			_JnlNote:= 'Retur Jual '||fBPName(_CustID)||':'||_SRetNo||E' \nKeterangan : '|| _SRetNote;
		END IF;
		
		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo1, 'SRET', _SRetDate, 'SRET', _SRetNo,
			_JnlNote,TRUE,TRUE,cast(0 as NUMERIC(19,4)),
			_CrtBy,_CrtDate,_UpdBy,_UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _Excrate, _FisRate
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		------------------------ Ayat HPP ----------------		
		_JnlNo2:=fTransNoGet('JURNAL');
		_JnlNote:= 'HPP Retur Jual '||fBPName(_CustID)||':'||_SRetNo;
		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo2, 'SRET', _SRetDate, 'SRET',_SRetNo,
			_JnlNote,TRUE,TRUE,cast(0 as NUMERIC(19,4)),
			_CrtBy,_CrtDate,_UpdBy,_UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _Excrate, _FisRate			
			);			
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
		
		IF _TermType='C' THEN
			_AccAsset:=fAccCash(_CashID, _InOut);
/*
		ELSIF (_ConsNo IS NOT NULL) THEN
			_AccAsset:=fAccCash(_CashID, _InOut);		
*/
		ELSE
			_AccAsset:=fBpAcc(_CustID, _CrcID, 'AR');
		END IF;

		_Count1:=0;
		_Count2:=0;

		SELECT INTO 
			_TotalBasePrice 
			Round(Sum(Qty*(BasePrice-COALESCE(DiscAmt,0))*_ExcRate),_Round) 
		FROM SRetD WHERE SRetNo=_SRetNo AND (fItemAcc(ItemID, 'SRET') IS NOT NULL AND fItemAcc(ItemID, 'SRET')<>'');

		_SelisihBasePrice:=(_Subtotal*_ExcRate)-_TotalBasePrice;
		_i:=0;
		--******************* JURNAL SRet: DEBET *************************--
		OPEN cSRetAcc FOR
			SELECT fItemAcc(ItemID, 'SRET'), DeptID, PrjID, SUM(basesubtotal), SUM(subtotal) 
			FROM SRetD 
			WHERE SRetNo=_SRetNo 
			GROUP BY fItemAcc(ItemID, 'SRET'), DeptID, PrjID;
		LOOP
			FETCH cSRetAcc INTO _AccSRet, _DeptID, _PrjID, _DBSubtotal, _DSubtotal;
			EXIT WHEN NOT FOUND;

			IF (_i=0) THEN
				_AccAmt:=_AccAmt+_SelisihBasePrice;
			END IF;
			_i:=_i+1;
		
			_Count1:=_Count1+1;		
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccSRet, 'D', _DBSubtotal, _DeptID, _PrjID, _DSubtotal);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cSRetAcc;


		_Disc:=COALESCE(_Disc, 0);
		IF (_Disc<>0) THEN
			IF (_AccDisc IS NULL) THEN
				_AccDisc:=fAccDefault('SALEDISC');
			END IF;

			_Count1:=_Count1+1;		
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccDisc, 'D', ROUND(_Disc*_ExcRate*-1, _Round), NULL, NULL,_Disc*-1);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		SELECT INTO _TotalTax SUM(ROUND(d.taxamt*Qty*_ExcRate, _RoundTax)) 
		FROM SRetD d JOIN tax ON d.taxid=tax.taxid
			WHERE d.SRetNo=_SRetNo and d.taxid IS NOT NULL;

		_SelisihTax:=(_MTaxAmt*_ExcRate)-_TotalTax;
		_i:=0;

		/*POSTING TAX DI GRUP IN SESUAI PAJAK TIAP2 ITEM*/
		OPEN cTax FOR
			SELECT SUM(d.basftotaltaxamt), SUM(d.totaltaxamt), accsaletax, deptid, prjid
			FROM SRetD d JOIN tax ON d.taxid=tax.taxid
			WHERE d.SRetNo=_SRetNo and d.taxid IS NOT NULL
			GROUP BY accsaletax, deptid, prjid;
		LOOP
			FETCH cTax INTO _BTotalTaxAmt, _TotalTaxAmt, _AccSRetTax, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;

			IF (_i=0) then
				_SRetDTaxAmt:=_SRetDTaxAmt+_SelisihTax;
			end if;
			_i:=_i+1;

			_Count1:=_Count1+1;	
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccSRetTax, 'D', _BTotalTaxAmt, _DeptID, _PrjID, _TotalTaxAmt);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cTax;
		/*IF (_MTaxAmt<>0) THEN
			_Count1:=_Count1+1;	
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, fAccDefault('SALETAX'), 'D', _MTaxAmt*_ExcRate, NULL, NULL);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;*/

/** ==================== LABA RUGI SELISIH KURS ================================ */
		_RateLR:=(_BTotal+_MBDiscAmt)-(_BSubtotal+_MBFTaxAmt);
		IF (_RateLR!=0) THEN
			_Count1:=_Count1+1;
			_AccRateLR:=fAccDefault('PKKURS');
			IF (_AccRateLR IS NULL OR _AccRateLR='') THEN
				_retval.ErrNo:=50555;
				_retval.ErrMsg:='Akun PK Selisih Kurs belum diset.'||E'\n'||'Silahkan disetting pada form Setting Akun';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccRateLR, 'D', _RateLR, _DeptID, _PrjID, NULL, _RateLR/_ExcRate);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		
		IF (_Freight<>0) THEN
			IF (_AccFreight IS NULL) THEN
				_AccFreight:=fAccDefault('SALEFREIGHT');
			END IF;
			_Count1:=_Count1+1;		
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccFreight, 'D', _Freight*_ExcRate, NULL, NULL,_Freight);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
		IF (_Others<>0) THEN
			_Count1:=_Count1+1;		
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, fAccDefault('PKROUND'), 'D', _Others*_ExcRate, NULL, NULL,_Others);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		/******************* JURNAL SRet: KREDIT ***************************/
		_Count1:=_Count1+1;		
		SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccAsset, 'K', _BTotal, _DeptID, _PrjID, _Total);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
		/*--------------- proses jurnal untuk HPP: DEBET ----------------------------*/
		IF (_countDeli>0) THEN	
			
			_Count2:=_Count2+1;
			SELECT INTO _UBDeliAmt SUM(ROUND(Cost*Qty, _Round)) FROM SRetD WHERE SRetNo=_SRetNo AND DeliNo IS NOT NULL AND DeliNo<>'';

			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo2, _Count2, fAccDefault('UBSRet'), 'D', _UBDeliAmt, _DeptID, _PrjID, _UBDeliAmt);
			
		ELSE
			
			OPEN cSRetAcc FOR
				SELECT fItemAcc(d.ItemID, 'INV'), d.DeptID, d.PrjID, SUM(ROUND(d.Qty*d.Cost, _Round))
				FROM StockAD d
				JOIN StockA s ON s.StockANo=d.StockANo
				LEFT JOIN item i ON i.ItemID=d.ItemID
				WHERE s.RefNo=_SRetNo AND s.RefType='SRET'
				GROUP BY fItemAcc(d.ItemID, 'INV'), d.DeptID, d.PrjID;
			LOOP
				FETCH cSRetAcc INTO _AccInv, _DeptID, _PrjID, _AccAmt;
				EXIT WHEN NOT FOUND;
				
				_Count2:=_Count2+1;		
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo2, _Count2, _AccInv, 'D', _AccAmt, _DeptID, _PrjID, _AccAmt);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END LOOP;
			CLOSE cSRetAcc;		
		END IF;
		
		/*--------------- proses jurnal untuk HPP: KREDIT ----------------------------*/
		OPEN cSRetAcc FOR
			SELECT fItemAcc(d.ItemID, 'COST'), d.DeptID, d.PrjID, SUM(ROUND(d.Qty*d.Cost, _Round))
			FROM StockAD d
			JOIN StockA s ON s.StockANo=d.StockANo
			LEFT JOIN item i on i.ItemID=d.ItemID
			WHERE s.RefNo=_SRetNo AND s.RefType='SRET'
			GROUP BY fItemAcc(d.ItemID, 'COST'), d.DeptID, d.PrjID;			
		LOOP
			FETCH cSRetAcc INTO _AccCOGS, _DeptID, _PrjID, _AccAmt;
			EXIT WHEN NOT FOUND;
			
			_Count2:=_Count2+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo2, _Count2, _AccCOGS, 'K', _AccAmt, _DeptID, _PrjID,_AccAmt);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cSRetAcc;

		/*---------- execute spJnlNew --------------*/
		SELECT INTO _spret * FROM spJnl_New(_JnlNo1);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF (_Count2>1) THEN
		SELECT INTO _spret * FROM spJnl_New(_JnlNo2);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		ELSE
			DELETE FROM jnl WHERE jnlno=_JnlNo2;
		END IF;

	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


